@@ -44,7 +44,8 @@ void make_dev_symlink(char *dev)
 		strcpy(new+7, new+8);
 	else
 		new[7] = '_';
-	symlink(dev+5, new);
+	if (symlink(dev+5, new))
+		perror(new);
 }
 
 
@@ -134,10 +135,6 @@ int open_mddev(char *dev, int autof)
 				dev);
 			return -1;
 		}
-		if (autof == 2 && stb.st_mode == 0 && !is_standard(dev, NULL)) {
-			fprintf(stderr, Name ": --auto=yes requires a 'standard' md device name, not %s\n", dev);
-			return -1;
-		}
 		/* check major number is correct */
 		num = -1;
 		std = is_standard(dev, &num);
@@ -145,19 +142,23 @@ int open_mddev(char *dev, int autof)
 		switch(autof) {
 		case 2: /* only create is_standard names */
 			if (!std && !stb.st_mode) {
-				fprintf(stderr, Name ": --auto=yes requires a 'standard' md device name, not %s\n", dev);
+				fprintf(stderr, Name
+			": %s does not exist and is not a 'standard' name "
+			"so it cannot be created\n", dev);
 				return -1;
 			}
 			break;
 		case 3: /* create md, reject std>0 */
 			if (std > 0) {
-				fprintf(stderr, Name ": that --auto option not compatable with device named %s\n", dev);
+				fprintf(stderr, Name ": that --auto option "
+				"not compatable with device named %s\n", dev);
 				return -1;
 			}
 			break;
 		case 4: /* create mdp, reject std<0 */
 			if (std < 0) {
-				fprintf(stderr, Name ": that --auto option not compatable with device named %s\n", dev);
+				fprintf(stderr, Name ": that --auto option "
+				"not compatable with device named %s\n", dev);
 				return -1;
 			}
 			break;
@@ -171,7 +172,6 @@ int open_mddev(char *dev, int autof)
 		if (stb.st_mode && major(stb.st_rdev) != major)
 			must_remove = 1;
 		if (stb.st_mode && !must_remove) {
-			mdu_array_info_t array;
 			/* looks ok, see if it is available */
 			mdfd = open(dev, O_RDWR, 0);
 			if (mdfd < 0) {
@@ -184,17 +184,9 @@ int open_mddev(char *dev, int autof)
 				close(mdfd);
 				return -1;
 			}
-			if (ioctl(mdfd, GET_ARRAY_INFO, &array)==0) {
-				/* already active */
-				close(mdfd);
-				fprintf(stderr, Name ": %s is already active.\n",
-					dev);
-				return -1;
-			} else {
-				if (major != MD_MAJOR && parts > 0)
-					make_parts(dev, parts, ci->symlinks);
-				return mdfd;
-			}
+			if (major != MD_MAJOR && parts > 0)
+				make_parts(dev, parts, ci->symlinks);
+			return mdfd;
 		}
 		/* Ok, need to find a minor that is not in use.
 		 * If the device name is in a 'standard' format,
@@ -258,8 +250,10 @@ int open_mddev(char *dev, int autof)
 
 			if (strncmp(dev, "/dev/md/", 8) == 0) {
 				if (mkdir("/dev/md",0700)==0) {
-					chown("/dev/md", ci->uid, ci->gid);
-					chmod("/dev/md", ci->mode| ((ci->mode>>2) & 0111));
+					if (chown("/dev/md", ci->uid, ci->gid))
+						perror("chown /dev/md");
+					if (chmod("/dev/md", ci->mode| ((ci->mode>>2) & 0111)))
+						perror("chmod /dev/md");
 				}
 			}
 			if (mknod(dev, S_IFBLK|0600, makedev(major, minor))!= 0) {
@@ -298,3 +292,51 @@ int open_mddev(char *dev, int autof)
 	return mdfd;
 }
 
+
+int open_mddev_devnum(char *devname, int devnum, char *name, char *chosen_name)
+{
+	/* Open the md device with number 'devnum', possibly using 'devname',
+	 * possibly constructing a name with 'name', but in any case, copying
+	 * the name into 'chosen_name'
+	 */
+	int major, minor;
+	struct stat stb;
+
+	if (devname)
+		strcpy(chosen_name, devname);
+	else if (name && strchr(name,'/') == NULL) {
+		char *n = strchr(name, ':');
+		if (n) n++; else n = name;
+		if (isdigit(*n) && devnum < 0)
+			sprintf(chosen_name, "/dev/md/d%s", n);
+		else
+			sprintf(chosen_name, "/dev/md/%s", n);
+	} else {
+		if (devnum >= 0)
+			sprintf(chosen_name, "/dev/md%d", devnum);
+		else
+			sprintf(chosen_name, "/dev/md/d%d", -1-devnum);
+	}
+	if (devnum >= 0) {
+		major = MD_MAJOR;
+		minor = devnum;
+	} else {
+		major = get_mdp_major();
+		minor = (-1-devnum) << 6;
+	}
+	if (stat(chosen_name, &stb) == 0) {
+		/* It already exists.  Check it is right. */
+		if ( ! S_ISBLK(stb.st_mode) ||
+		     stb.st_rdev != makedev(major, minor)) {
+			errno = EEXIST;
+			return -1;
+		}
+	} else {
+		if (mknod(chosen_name, S_IFBLK | 0600,
+			  makedev(major, minor)) != 0) {
+			return -1;
+		}
+		/* FIXME chown/chmod ?? */
+	}
+	return open(chosen_name, O_RDWR);
+}
